
#include "aspell/data.hh"
#include "language.hh"
#include "const_string.hh"

void WordList_Base::attach(const SC_Language &l) {
  if (lang_ && const_string(l.name()) != lang_->name())
    throw ae_mismatched_lang(l.name(), this);
  if (!lang_) lang_ = l.clone();
  ++attach_count_;
}

void WordList_Base::detach() {
  --attach_count_;
}

WordList_Base::WordList_Base()
  : lang_(0), attach_count_(0), del_lang_(false) {}

WordList_Base::~WordList_Base() {
  if (del_lang_) delete lang_;
}

bool WordList_Base::is_attached() const {
  return attach_count_;
}

const char * WordList_Base::lang_name() const {
  return lang_->name();
}

void WordList_Base::check_lang(const string &l) {
  if (l != lang_->name()) 
    throw ae_mismatched_lang(l, this);
}

void WordList_Base::set_check_lang (const string &l, 
				    const string &lang_dir) {
  if (!is_attached()) {
    delete lang_;
    lang_ = SC_Languages(l,lang_dir);
    del_lang_ = true;
    if (!lang_)
      throw ae_unknown_lang(l, this);
  } else {
    if (l != lang_->name()) 
      throw ae_mismatched_lang(l, this);
  }
}




