
#include "aspell/aspell.hh"
#include "aspell/data.hh"
#include "language.hh"

//
// data_access functions
//

//const aspell::WordList aspell::empty_list;

aspell::aspell(const string &m, const string &p, const string &r, bool ir)
  : lang_(0), del_lang(true), ignore_repl(ir)
{
  try {
    {
      StandardWL_Base * temp = new ReadOnlyWL;
      temp->load(m);
      change_id(attach(temp), main_id)->own(true);
    } {
      StandardWL_Base * temp = new WritableWL;
      change_id(attach(temp), personal_id)->own(true);
      try {temp->load(".aspell");} catch (ae_cant_read&) {}
    } {
      change_id(attach(new WritableWL), session_id)->own(true);
    } {
      ReplacementsWL_Base * temp = new WritableReplList;
      change_id(attach(temp), per_repl_id)->own(true);
      try {temp->load(".aspell");} catch (ae_cant_read&) {}
    }
  } catch (...) {
    destroy();
    throw;
  }
}

aspell::aspell(): lang_(0), del_lang(true), ignore_repl(true)
{}

aspell::~aspell() {
  destroy();
}

const char * aspell::lang_name() const {
  return lang_->name();
}

string aspell::to_lower(const string &word) const {
  return lang_->to_lower(word);
}

const char * aspell::alpha_map() const {
  return lang_->alpha_map();
}

#if 0
const SC_Error aspell::set_check_lang (const string &l, 
				       const string &lang_dir) {
  if (!lang) {
    lang = SC_Languages(l,lang_dir);
    del_lang = true;
    if (!lang) {
      error_ = Error("", l, Error::unknown_lang);
      return error_;
    }
  } else {
    if (l != lang->name()) {
      error_ = Error("",l, Error::mismatched_lang);
      return error_;
    }
  }
  return Error();
}

#endif

void aspell::add_personal(const string &word) {
  WordLists::iterator i = locate_wl(personal_id);
  if (i != word_lists_.end())
    static_cast<WritableWL_Base *>(i->wl())->add(word);
}

void aspell::add_session(const string &word) {
  WordLists::iterator i = locate_wl(session_id);
  if (i != word_lists_.end())
    static_cast<WritableWL_Base *>(i->wl())->add(word);
}

void aspell::clear_session() {
  WordLists::iterator i = locate_wl(session_id);
  if (i != word_lists_.end())
    static_cast<WritableWL_Base *>(i->wl())->clear();
}

bool aspell::ignore_replacements() {
  return ignore_repl;
}

bool aspell::ignore_replacements(bool r) {
  bool temp = ignore_repl;
  ignore_repl = r;
  return temp;
}

void aspell::store_replacement(const string& mis, const string &cor,
			       bool memory) {
  WordLists::iterator i = locate_wl(per_repl_id);
  if (!ignore_repl && i != word_lists_.end()) {
    string::size_type pos;
    Suggestions sugs = suggest(mis);
    if (pos = cor.find(' '), 
	pos == string::npos 
	? check(cor)
	: (check(cor.substr(0,pos)) && check(cor.substr(pos+1))) )
      { // cor is a correct spelling
	
	if (sugs.empty() || cor != sugs[0])
	  static_cast<ReplacementsWL_Base *>(i->wl())->add(mis, cor);

	if (memory && prev_cor_repl_ == mis) 
	  store_replacement(prev_mis_repl_, cor, false);
	
      } else { // cor is not a correct spelling
	
	if (memory) {
	  if (prev_cor_repl_ != mis)
	    prev_mis_repl_ = mis;
	  prev_cor_repl_ = cor;
	}
      }
  }
}

//
// simple functions
//

inline bool aspell::check_notrim(const string &word) const {
  if (check_raw(word)) return true;
  else return check_raw(lang_->to_lower(word));
}

inline bool aspell::check_notrim(const char *word) const {
  if (check_raw(word)) return true; 
  else return check_raw(lang_->to_lower(word));
}

bool aspell::check(const string &word) const {
  if (check_notrim(word)) return true;
  return lang_->trim_n_try(*this, word);
}

bool aspell::check(const char *word) const {
  if (check_notrim(word)) return true;
  else return lang_->trim_n_try(*this, word);
}

const string aspell::to_soundslike(const string &word) const {
  return lang_->to_soundslike(word);
}

const aspell::SWordList aspell::soundslike(const string &word) const {
  return words_w_soundslike(lang_->to_soundslike(word));
}

const string aspell::to_phoneme(const string &word) const {
  return lang_->to_phoneme(word);
}

string ae_already_have_type::message() const {
  return "Sorry, I already have that type";
}

string ae_word_list_not_compellable::message() const {
  return "The current word list is not compatible with the requested id";
}
